@@ -1,10 +1,23 @@
-""" Inspired by https://github.com/warner/python-versioneer. """
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
+""" flowtool-versioning cmdclass file. flowtool-versioning provides an
+    automatic versioning system based on git tags. This is the command
+    set, that it brings to add capabilities to your setup.py file to
+    deal with git tags and handle the versioning. See the link for more
+    information:
+
+        https://github.com/isnok/py-flowlib
+"""
+
+# TODO: make these files pass pylint regularly
+# pylint: disable=E0401,E1101,E0611
+
 import os
 from os.path import join, exists, isfile, isdir, dirname, basename
 try:
-    import configparser
+    from configparser import ConfigParser
 except:
-    import ConfigParser as configparser
+    from ConfigParser import ConfigParser
 
 def find_parent_containing(name, path=None, check='exists'):
     """ Return the nearest directory in the parent dirs of path,
@@ -13,7 +26,7 @@ def find_parent_containing(name, path=None, check='exists'):
         and isdir.
     """
 
-    current = os.getcwd() if path is None else path
+    current = os.path.dirname(__file__) if path is None else path
 
     if check == 'exists':
         check = exists
@@ -36,7 +49,7 @@ def read_config(*filenames):
     # configparser.NoSectionError (if it lacks a [versioneer] section), or
     # configparser.NoOptionError (if it lacks "VCS="). See the docstring at
     # the top of versioneer.py for instructions on writing your setup.cfg .
-    parser = configparser.ConfigParser()
+    parser = ConfigParser()
     parser.read(filenames)
     return parser
 
@@ -45,7 +58,6 @@ setup_cfg = join(
     'setup.cfg',
 )
 parser = read_config(setup_cfg)
-
 source_versionfile = parser.get('versioning', 'source_versionfile')
 
 def import_file(name, path):
@@ -71,8 +83,15 @@ def import_file(name, path):
     except:
         pass
 
-version_in_git = import_file('versions', source_versionfile)
-get_version = version_in_git.get_version
+if __name__ != 'flowtool_versioning.dropins.cmdclass':
+
+    version_in_git = import_file('versions', source_versionfile)
+    if version_in_git:
+        get_version = version_in_git.get_version
+    else:
+        print("== Warning: source_versionfile %s could not be imported. (no tags found?)" % source_versionfile)
+        get_version = lambda: 'versionfile_not_installed'
+
 
 def build_versionfile():
     if parser.has_option('versioning', 'build_versionfile'):
@@ -82,15 +101,6 @@ def build_versionfile():
 
 #print(build_versionfile())
 
-def install_versionfile(to_file):
-    import flowtool_python.versioning
-    versionfile = join(
-        dirname(flowtool_python.versioning.__file__),
-        'version.py'
-    )
-    with open(versionfile, 'r') as f_in, open(to_file, 'w') as f_out:
-        f_out.write(f_in.read())
-
 
 def render_versionfile():
     print(version_in_git.render_versionfile())
@@ -114,8 +124,34 @@ class cmd_version_info(Command):
         print('== Version-Config (setup.cfg):\n%s' % pformat(dict(parser.items('versioning'))))
         print('== Version-Info:\n%s' % pformat(version_in_git.VERSION_INFO))
 
-class cmd_versioning_update(Command):
-    description = "show versioning configuration and current project version"
+def bump_version(info):
+    if 'dev_release' in info:
+        info['dev_release'] += 1
+    elif 'post_release' in info:
+        info['post_release'] += 1
+    elif 'pre_release' in info:
+        stage, number = info['pre_release']
+        info['pre_release'] = (stage, number + 1)
+    else:
+        segments = info['release']
+        info['release'] = segments[:-1] + (segments[-1] + 1,)
+    return info
+
+
+def render_bumped(**kwd):
+    normalized = '.'.join(map(str, kwd['release']))
+    if 'pre_release' in kwd:
+        normalized += '%s%s' % kwd['pre_release']
+    if 'post_release' in kwd:
+        normalized += '.post' + str(kwd['post_release'])
+    if 'dev_release' in kwd:
+        normalized += '.dev' + str(kwd['dev_release'])
+    if 'epoch' in kwd:
+        normalized = '{}!{}'.format(kwd['epoch'], normalized)
+    return normalized
+
+class cmd_version_bump(Command):
+    description = "bump the (pep440) version by adding one to the smallest version component"
     user_options = []
     boolean_options = []
 
@@ -127,8 +163,33 @@ class cmd_versioning_update(Command):
 
     def run(self):
         from pprint import pformat
+        vcs_info = version_in_git.VERSION_INFO['vcs_info']
+        tag_info = bump_version(vcs_info['tag_version'])
+        print('== Current Version:\n%s' % pformat(tag_info))
+        if vcs_info['dirt']:
+            print("==> Auto bump aborted due to dirty git repository.")
+            sys.exit(1)
+        tag = vcs_info['prefix'] + render_bumped(**tag_info)
+        print('== Tagging: %s' % tag)
+        os.system('git tag ' + tag)
+
+class cmd_update_versionfile(Command):
+    description = "update the versioning"
+    user_options = []
+    boolean_options = []
+
+    def initialize_options(self):
+        pass
+
+    def finalize_options(self):
+        pass
+
+    def run(self):
         print('== Updating file:\n%s' % source_versionfile)
-        install_versionfile(source_versionfile)
+        from flowtool_versioning.dropins import version
+        versionfile = version.__file__
+        with open(versionfile, 'r') as f_in, open(source_versionfile, 'w') as f_out:
+            f_out.write(f_in.read())
 
 
 if "setuptools" in sys.modules:
@@ -195,12 +256,46 @@ class cmd_sdist(_sdist):
         with open(target_versionfile, 'w') as fh:
             fh.write(version_in_git.render_static_file())
 
+from distutils.command.upload import upload as _upload
+
+class cmd_upload(_upload):
+
+    description="Do the normal upload, but prevent pushing with Python2."
+
+    def run(self):
+        if sys.version_info.major == 3:
+            return _upload.run(self)
+        print('==> For backwards compatibility you should only upload packages built with Python 3 to PyPI.')
+        sys.exit(1)
+
+class cmd_release(cmd_upload):
+
+    description="Do the protected upload, but push git things first"
+
+    def run(self):
+        print("=== git push")
+        os.system('git push')
+        print("=== pushing tags also")
+        os.system('git push --tags')
+        return cmd_upload.run(self)
+
+
 def get_cmdclass():
     """Return the custom setuptools/distutils subclasses."""
     cmds = dict(
-        version_info=cmd_version_info,
-        versioning_update=cmd_versioning_update,
+        version=cmd_version_info,
+        #versioning_update=cmd_versioning_update,
+        bump=cmd_version_bump,
         build_py=cmd_build_py,
         sdist=cmd_sdist,
+        upload=cmd_upload,
+        release=cmd_release,
     )
     return cmds
+
+
+def main():
+    print(get_version())
+
+if __name__ == '__main__':
+    main()
